﻿using ProjectMotorboat.Drownings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMotorboat.CollectionGenericObjects;

public class StorageCollection<T>
    where T : DrawningBoat
{

    private Dictionary<string, ICollectionGenericObjects<T>> _storages;

    public List<string> Keys => _storages.Keys.ToList();

    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }

    
    public void AddCollection(string name, CollectionType collectionType)
    {
        // TODO проверка, что name не пустой и нет в словаре записи с таким ключом
        // TODO Прописать логику для добавления
        if (name == null || _storages.ContainsKey(name)) { return; }
        switch (collectionType)

        {
            case CollectionType.None:
                return;
            case CollectionType.Massive:
                _storages[name] = new MassiveGenericObjects<T>();
                return;
            case CollectionType.List:
                _storages[name] = new ListGenericObjects<T>();
                return;
        }
    }

   
    public void DelCollection(string name)
    {
        // TODO Прописать логику для удаления коллекции
        if (_storages.ContainsKey(name))
            _storages.Remove(name);
    }

   
    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            // TODO Продумать логику получения объекта
            if (name == null || !_storages.ContainsKey(name)) { return null; }
            return _storages[name];
        }
    }

    private readonly string _collectionKey = "CollectionsStorage";

    private readonly string _separatorForKeyValue = "|";

    private readonly string _separatorItems = ";";

    public bool SaveData(string filname)
    {
        if (_storages.Count == 0)
        {
            return false;
        }

        if (File.Exists(filname))
        {
            File.Delete(filname);
        }

        if (File.Exists(filname))
        {
            File.Delete(filname);
        }

        StringBuilder sb = new();

        sb.Append(_collectionKey);
        foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
        {
            sb.Append(Environment.NewLine);
            // не сохраняем пустые коллекции
            if (value.Value.Count == 0)
            {
                continue;
            }

            sb.Append(value.Key);
            sb.Append(_separatorForKeyValue);
            sb.Append(value.Value.GetCollectionType);
            sb.Append(_separatorForKeyValue);
            sb.Append(value.Value.MaxCount);
            sb.Append(_separatorForKeyValue);

            foreach (T? item in value.Value.GetItems())
            {
                string data = item?.GetDataForSave() ?? string.Empty;
                if (string.IsNullOrEmpty(data))
                {
                    continue;
                }

                sb.Append(data);
                sb.Append(_separatorItems);
            }
        }

        using FileStream fs = new(filname, FileMode.Create);
        byte[] info = new UTF8Encoding(true).GetBytes(sb.ToString());
        fs.Write(info, 0, info.Length);
        return true;
    }

    public bool LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            return false;
        }

        string bufferTextFromFile = "";
        using (FileStream fs = new(filename, FileMode.Open))
        {
            byte[] b = new byte[fs.Length];
            UTF8Encoding temp = new(true);
            while (fs.Read(b, 0, b.Length) > 0)
            {
                bufferTextFromFile += temp.GetString(b);
            }
        }

        string[] strs = bufferTextFromFile.Split(new char[] { '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);
        if (strs == null || strs.Length == 0)
        {
            return false;
        }
        if (!strs[0].Equals(_collectionKey))
        {
            //если нет такой записи, то это не те файлы
            return false;
        }

        _storages.Clear();
        foreach (string data in strs)
        {
            string[] record = data.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
            if (record.Length != 4)
            {
                continue;
            }

            CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
            ICollectionGenericObjects<T> collection = StorageCollection<T>.CreateCollection(collectionType);
            if (collection == null)
            {
                return false;
            }
            collection.MaxCount = Convert.ToInt32(record[2]);

            string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
            foreach (string elem in set)
            {
                if (elem?.CreateDrawningBoat() is T warship)
                {
                    if (collection.Insert(warship) == -1)
                    {
                        return false;
                    }
                }
            }

            _storages.Add(record[0], collection);
        }
        return true;
    }
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}